package pl.edu.agh.scaa.solver;


import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import pl.edu.agh.scaa.gui.ExecutorParameters;
import pl.edu.agh.scaa.gui.MainWindow;

public class Executor extends Thread {	
    	private int n = 64;
	private double alpha = 0.5;
	private double delta = 0.001;	
	private int stepNum = 300;
	private MainWindow mainWindow = null;
	private List<Double> solution = null;
	private List<Vertex> leafs = null;
	private boolean stop = true;

	public Executor(MainWindow mainWindow, ExecutorParameters executorParameters) {
		this.mainWindow = mainWindow;
		this.stepNum = executorParameters.getStepNum();
		this.n = executorParameters.getN();
		this.delta = executorParameters.getDelta();
		this.alpha = executorParameters.getAlpha();
	}

	public Executor() {
	}

	private synchronized Vertex makeFloodMESTree(int n) {
		Queue<Vertex> q2 = new LinkedList<Vertex>();
		leafs = new LinkedList<Vertex>();
		Queue<Vertex> tmpQ;
		Counter counter = new Counter(this);
		counter.setCounter(n);
		Queue<Vertex> q = new LinkedList<Vertex>();
		Vertex v1 = new Vertex(null, null, null, "leaf");
		Vertex v, v2;
		AFloodMES a1 = new AFloodMES(v1, counter, 0, solution, alpha, delta);
		a1.start();
		leafs.add(v1);
		q.offer(v1);
		for (int i = 1; i < n - 1; i++) {
			v = new Vertex(null, null, null, "leaf");
			AFloodMES a = new AFloodMES(v, counter, i, solution, alpha, delta);
			a.start();
			leafs.add(v);
			q.offer(v);
		}
		Vertex vn = new Vertex(null, null, null, "leaf");
		AFloodMES an = new AFloodMES(vn, counter, n - 1, solution, alpha, delta);
		an.start();
		leafs.add(vn);
		q.offer(vn);
		while (q.size() > 2) {
			while (q.size() >= 2) {
				v1 = q.poll();
				v2 = q.poll();
				v = new Vertex(v1, v2, null, "node");
				v1.set_parent(v);
				v2.set_parent(v);
				q2.offer(v);
			}
			if (q.size() == 1) {
				v = q.poll();
				q2.offer(v);
			}
			tmpQ = q;
			q = q2;
			q2 = tmpQ;
		}
		v1 = q.poll();
		v2 = q.poll();
		v = new Vertex(v1, v2, null, "root");
		v1.set_parent(v);
		v2.set_parent(v);
		counter.release();
		return v;
	}

	public void applyA2E(Vertex v) {
		Counter counter = new Counter(this);
		if ("leaf" == v.m_label)
			return;
		applyA2E(v.m_left);
		applyA2E(v.m_right);
		counter.setCounter(1);
		A2Prim a = new A2Prim(v, counter);
		a.start();
		counter.release();

		counter.setCounter(1);
		E2 e = new E2(v, counter);
		e.start();
		counter.release();
	}

	public void applyBS(Vertex v) {
		Counter counter = new Counter(this);
		if ("root" == v.m_label) {
			counter.setCounter(1);
			BS bs = new BS(v, counter);
			bs.start();
			counter.release();
			applyBS(v.m_left);
			applyBS(v.m_right);
			return;
		}
		if ("leaf" == v.m_label)
			return;
		counter.setCounter(1);
		BSS bss = new BSS(v, counter);
		bss.start();
		counter.release();
		counter.setCounter(1);
		BS bs = new BS(v, counter);
		bs.start();
		counter.release();
		applyBS(v.m_left);
		applyBS(v.m_right);

	}

	private void initSolution(int n) {
		Terrain z = new Terrain();
		solution = new LinkedList<Double>();
		for (int i = 0; i <= n; i++) {
			solution.add(z.getTerrain((double)i/(double)n));
		}
	}

	private void updateRec(int n, Vertex v, ArrayList<Double> sol) {
		if ("leaf" == v.m_label) {
			sol.add(v.m_b[1]);
			if (sol.size() == n) {
				sol.add(v.m_b[2]);
			}
			return;
		}
		updateRec(n, v.m_left, sol);
		updateRec(n, v.m_right, sol);
	}

	private void update(int n, Vertex v) {
		ArrayList<Double> newSol = new ArrayList<Double>();
		updateRec(n, v, newSol);
		solution = newSol;
	}

	private void step(int n) {
		Counter counter = new Counter(this);
		//Vertex S = makeMESTree(n);
		Vertex S = makeFloodMESTree(n);
		applyA2E(S);

		counter.setCounter(1);
		ERoot eRoot = new ERoot(S, counter);
		eRoot.start();
		counter.release();
		applyBS(S);

		update(n, S);
	}
	
	public void stopSimulation() {
		stop = true;
	}

	public void run() {
		stop = false;
		//generateStartingSolution(n);
		initSolution(n);
		Rain.it=0;
		for (int i = 0; i < stepNum; i++) {
			if (stop) {
				break;
			}
			step(n);
			if (mainWindow != null) {
				List<Double> stepResult = new ArrayList<Double>(solution.size());
				for (double d : solution) {
					stepResult.add(d);
					//System.out.println(d);
				}
				mainWindow.updateData(i, stepResult);
			}
		}
		if (mainWindow != null) {
			mainWindow.computationFinished();
		}
	}

	public synchronized void printResult(Vertex v) {
		if ("leaf" == v.m_label) {
			System.out.println(v.m_b[1] + " ");
			return;
		}
		printResult(v.m_left);
		printResult(v.m_right);
	}
}
