package pl.edu.agh.scaa.gui;

import static java.lang.Math.max;
import static java.lang.Math.min;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import pl.edu.agh.scaa.solver.Executor;


public class MainWindow extends JFrame {

	private static final long serialVersionUID = 2740437090361841747L;

	private XYSeries series;
	private XYSeries terrain;
	
	private XYSeriesCollection dataset = new XYSeriesCollection();

	private Executor executor;

	private final MenuPanel menuPanel;

	private JFreeChart chart;

	public MainWindow(String name) {
		super(name);
		menuPanel = new MenuPanel(this);
		createWindow();
		
//ZMIANA TERENU, DRUGA ZMIANA W KLASIE TERRAIN
		
		terrain.add(0,1);
		terrain.add(0.5,0);
		terrain.add(1,1);
		
//		terrain.add(0,1);
//		terrain.add(0.5,0);
//		terrain.add(0.6,0.2);
//		terrain.add(1,-0.6);
	}

	public void updateData(int stepNum, List<Double> newData) {
		dataset.removeAllSeries();
		series.clear();
		double partitionLength = 1.0 / (newData.size() - 1);
		for (int i = 0; i < newData.size(); i++) {
			series.add(partitionLength * i, newData.get(i));
		}
		
		dataset.addSeries(terrain);
		dataset.addSeries(series);
		updateRangeAxis();
		menuPanel.setCurrentState(stepNum + "");
	}

	public void startComputation() {
		stopComputation();
		resetPlot();
		executor = new Executor(this, menuPanel.getExecutorParameters());
		executor.start();
	}

	public void stopComputation() {
		if (executor != null) {
			executor.stopSimulation();
		}
	}

	public void computationFinished() {
		menuPanel.setCurrentState("Computation finished");
	}

	private void createWindow() {
		JPanel panel = new JPanel();
		JPanel chartPanel = new JPanel();
		JPanel mPanel = new JPanel();
		chartPanel.add(getChartPanel());
		mPanel.add(menuPanel);
		mPanel.setBackground(Color.white);
		chartPanel.setBackground(Color.white);
		this.setBackground(Color.white);
		panel.setLayout(new BorderLayout());
		panel.add(mPanel, BorderLayout.LINE_START);
		panel.add(chartPanel, BorderLayout.CENTER);
		add(panel);
		setSize(900, 480);
		setResizable(false);
		setLocationRelativeTo(getRootPane());
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setVisible(true);
		//pack();
	}

	private ChartPanel getChartPanel() {
		series = new XYSeries("flood");
		terrain = new XYSeries("terrain");
		chart = ChartFactory.createXYLineChart("", "", "", dataset,
				PlotOrientation.VERTICAL, true, true, false);
		
		//Obrazek ta;
		
//		BufferedImage img = null;
//		try {
//		    img = ImageIO.read(new File("blue-sky2.jpg"));
//		    
//		    chart.getPlot().setBackgroundImage(img);
//		} catch (IOException e) {
//		}
		
		ChartPanel panel = new ChartPanel(chart);
		((XYPlot) chart.getPlot()).getDomainAxis().setRange(0.0, 1.0);
		((XYPlot) chart.getPlot()).getRangeAxis().setRange(0.0, 1.5);
		((XYPlot) chart.getPlot()).getRenderer().setSeriesStroke(1, new BasicStroke(2.0f));
		((XYPlot) chart.getPlot()).getRenderer().setSeriesPaint(1, Color.BLUE);
		((XYPlot) chart.getPlot()).getRenderer().setSeriesStroke(0, new BasicStroke(3.5f));
		((XYPlot) chart.getPlot()).getRenderer().setSeriesPaint(0, Color.GREEN);		
		resetPlot();
		panel.setVisible(true);
		return panel;
	}

	private void resetPlot() {
		series.clear();
		series.add(0,1);
		series.add(0.5,0);
		series.add(1,1);
		updateRangeAxis();
	}

	private void updateRangeAxis() {
		ValueAxis valueAxis = ((XYPlot) chart.getPlot()).getRangeAxis();
		double upperBound = max(0.5, max(series.getMaxY() + 0.1, valueAxis.getUpperBound()));
		double lowerBound = min(-0.1, min(series.getMinY() - 0.1, valueAxis.getLowerBound()));
		valueAxis.setRange(lowerBound, upperBound);
	}
}
