package pl.edu.agh.scaa.solver;

import java.util.List;

/**
 * @(#)Production.java
 * 
 * 
 * @author
 * @version 1.00 2011/6/13
 */

abstract class Production extends Thread {

	Production(Vertex Vert, Counter Count) {
		m_vertex = Vert;
		m_counter = Count;
		m_drawer = new GraphDrawer();
	}

	// returns first vertex from the left
	abstract Vertex apply(Vertex v);

	// run the thread
	public void run() {
		// apply the production
		m_vertex = apply(m_vertex);
		// plot the graph
		m_drawer.draw(m_vertex);
		m_counter.dec();
	}

	// vertex where the production will be applied
	Vertex m_vertex;
	// graph drawer
	GraphDrawer m_drawer;
	// productions counter
	Counter m_counter;
}

class P1 extends Production {
	P1(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex S) {
		// System.out.println("p1");
		Vertex T1 = new Vertex(null, null, S, "T");
		Vertex T2 = new Vertex(null, null, S, "T");
		S.set_left(T1);
		S.set_right(T2);
		S.set_label("root");
		return S;
	}
}

class P2 extends Production {
	P2(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("p2");
		Vertex T1 = new Vertex(null, null, T, "T");
		Vertex T2 = new Vertex(null, null, T, "T");
		T.set_left(T1);
		T.set_right(T2);
		T.set_label("int");
		return T;
	}
}

class P3 extends Production {
	P3(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("p3");
		Vertex T1 = new Vertex(null, null, T, "leaf");
		Vertex T2 = new Vertex(null, null, T, "leaf");
		T.set_left(T1);
		T.set_right(T2);
		T.set_label("int");
		return T;
	}
}

class A1 extends Production {
	A1(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("A1");
		T.m_a[1][1] = 1.0;
		T.m_a[2][1] = 1.0;
		T.m_a[1][2] = 0.0;
		T.m_a[2][2] = -1.0;
		T.m_b[1] = 0.0;
		T.m_b[2] = 0.0;

		return T;
	}
}

class A extends Production {
	A(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("A");
		T.m_a[1][1] = -1.0;
		T.m_a[2][1] = 1.0;
		T.m_a[1][2] = 1.0;
		T.m_a[2][2] = -1.0;
		T.m_b[1] = 0.0;
		T.m_b[2] = 0.0;

		return T;
	}
}

class AN extends Production {
	AN(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("AN");
		T.m_a[1][1] = -1.0;
		T.m_a[2][1] = 0.0;
		T.m_a[1][2] = 1.0;
		T.m_a[2][2] = 1.0;
		T.m_b[1] = 0.0;
		T.m_b[2] = 1.0;

		return T;
	}
}

class A1MES extends Production {
	private int elementNumber;
	private List<Double> solution;
	private double alpha;
	private double delta;// time step
	private double d; 

	A1MES(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	A1MES(Vertex Vert, Counter Count, int elementNumber, List<Double> solution,
			double alpha, double delta, double elementSize) {
		super(Vert, Count);
		this.elementNumber = elementNumber;
		this.solution = solution;
		this.alpha = alpha;
		this.delta = delta;
		this.d =  elementSize;
	}

	Vertex apply(Vertex T) {
		T.setElementSize(d);
		// System.out.println("A1MES");
		T.m_a[1][1] = d / 3.0 + delta * alpha / d+delta*alpha;
		T.m_a[2][1] = d / 6.0 - delta * alpha / d;
		T.m_a[1][2] = d / 6 - delta * alpha / d;
		T.m_a[2][2] = d / 3.0 + delta * alpha / d;
		T.m_b[1] = solution.get(elementNumber)
				* d
				/ 3.0
				+ solution.get(elementNumber + 1)
				* d
				/ 6.0
				+ (delta * alpha - delta)
				* (solution.get(elementNumber) / d - solution.get(elementNumber + 1)
						/ d) -delta +delta*solution.get(elementNumber)*(alpha-1);
		T.m_b[2] = solution.get(elementNumber)
				* d
				/ 6.0
				+ solution.get(elementNumber + 1)
				* d
				/ 3.0
				+ (delta * alpha - delta)
				* (solution.get(elementNumber + 1) / d - solution.get(elementNumber)
						/ d);

		return T;
	}
}

class AMES extends Production {
	private int elementNumber;
	private List<Double> solution;
	private double alpha;
	private double delta;
	private double d;

	AMES(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	AMES(Vertex Vert, Counter Count, int elementNumber, List<Double> solution,
			double alpha, double delta, double elementSize) {
		super(Vert, Count);
		this.elementNumber = elementNumber;
		this.solution = solution;
		this.alpha = alpha;
		this.delta = delta;
		this.d = elementSize;
	}

	Vertex apply(Vertex T) {
		T.setElementSize(d);
		// System.out.println("A1MES");
		T.m_a[1][1] = d / 3.0 + delta * alpha / d;
		T.m_a[2][1] = d / 6.0 - delta * alpha / d;
		T.m_a[1][2] = d / 6 - delta * alpha / d;
		T.m_a[2][2] = d / 3.0 + delta * alpha / d;
		T.m_b[1] = solution.get(elementNumber)
				* d
				/ 3.0
				+ solution.get(elementNumber + 1)
				* d
				/ 6.0
				+ (delta * alpha - delta)
				* (solution.get(elementNumber) / d - solution.get(elementNumber + 1)
						/ d);
		T.m_b[2] = solution.get(elementNumber)
				* d
				/ 6.0
				+ solution.get(elementNumber + 1)
				* d
				/ 3.0
				+ (delta * alpha - delta)
				* (solution.get(elementNumber + 1) / d - solution.get(elementNumber)
						/ d);
		return T;
	}
}

class ANMES extends Production {
	private int elementNumber;
	private List<Double> solution;
	private double alpha;
	private double delta;
	private double d;

	ANMES(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	ANMES(Vertex Vert, Counter Count, int elementNumber, List<Double> solution,
			double alpha, double delta, double elementSize) {
		super(Vert, Count);
		this.elementNumber = elementNumber;
		this.solution = solution;
		this.alpha = alpha;
		this.delta = delta;
		this.d = elementSize;
	}

	Vertex apply(Vertex T) {
		T.setElementSize(d);
		// System.out.println("A1MES");
		T.m_a[1][1] = d / 3.0 + delta * alpha / d;
		T.m_a[2][1] = d / 6.0 - delta * alpha / d;
		T.m_a[1][2] = d / 6 - delta * alpha / d;
		T.m_a[2][2] = d / 3.0 + delta * alpha / d + delta*alpha;
		T.m_b[1] = solution.get(elementNumber)
				* d
				/ 3.0
				+ solution.get(elementNumber + 1)
				* d
				/ 6.0
				+ (delta * alpha - delta)
				* (solution.get(elementNumber) / d - solution.get(elementNumber + 1)
						/ d);
		T.m_b[2] = solution.get(elementNumber)
				* d
				/ 6.0
				+ solution.get(elementNumber + 1)
				* d
				/ 3.0
				+ (delta * alpha - delta)
				* (solution.get(elementNumber + 1) / d - solution.get(elementNumber)
						/ d) + delta+delta*solution.get(elementNumber+1)*(alpha-1);
		return T;
	}
}

class AFloodMES extends Production {
	private int elementNumber;
	private List<Double> solution;
	private double alpha;
	private double delta;

	AFloodMES(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	AFloodMES(Vertex Vert, Counter Count, int elementNumber, List<Double> solution,
			double alpha, double delta) {
		super(Vert, Count);
		this.elementNumber = elementNumber;
		this.solution = solution;
		this.alpha = alpha;
		this.delta = delta;
	}

	Vertex apply(Vertex T) {
		double d = 1.0 / (solution.size() - 1);
		double d3 = d / 3.0;
		double d6 = d / 6.0;
		double n = solution.size()-1.0;
		T.m_a[1][1] = d / 3.0;
		T.m_a[2][1] = d / 6.0;
		T.m_a[1][2] = d / 6.0;
		T.m_a[2][2] = d / 3.0;
		Rain f = new Rain();
		Terrain z = new Terrain();
		
		T.m_b[1] = solution.get(elementNumber)*d3 + solution.get(elementNumber+1)*d6;
		T.m_b[1] += delta*(f.getRain((double)elementNumber/n)*d/2.0+ f.getRain((double)(elementNumber+1)/n)*d/2.0);
		double line = -delta*(solution.get(elementNumber)/d - solution.get(elementNumber+1)/d);
		line *= Math.pow( Math.abs(solution.get(elementNumber)/2.0 + solution.get(elementNumber+1)/2.0 - z.getTerrain(((double)elementNumber+0.5)/n)) , 5.0/3.0);
		line /= Math.pow( Math.abs(-solution.get(elementNumber)/d + solution.get(elementNumber+1)/d) , 0.5);
		T.m_b[1] += line;
		
		T.m_b[2] = solution.get(elementNumber)*d6 + solution.get(elementNumber+1)*d3;
		T.m_b[2] += delta*(f.getRain((double)elementNumber/n) * d/2.0 + f.getRain((double)(elementNumber+1)/n)*d/2.0);
		line = -delta*(-solution.get(elementNumber)/d + solution.get(elementNumber+1)/d);
		line *= Math.pow( Math.abs(solution.get(elementNumber)/2.0 + solution.get(elementNumber+1)/2.0 - z.getTerrain(((double)elementNumber+0.5)/n)) , 5.0/3.0);
		line /= Math.pow( Math.abs(-solution.get(elementNumber)/d + solution.get(elementNumber+1)/d) , 0.5);		
		T.m_b[2] += line;
		
		return T;
		
	}
}


class A2 extends Production {
	A2(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("A2");
		T.m_a[0][0] = T.m_left.m_a[1][1];
		T.m_a[1][0] = T.m_left.m_a[2][1];
		T.m_a[2][0] = 0.0;
		T.m_a[0][1] = T.m_left.m_a[1][2];
		T.m_a[1][1] = T.m_left.m_a[2][2] + T.m_right.m_a[1][1];
		T.m_a[2][1] = T.m_right.m_a[2][1];
		T.m_a[0][2] = 0.0;
		T.m_a[1][2] = T.m_right.m_a[1][2];
		T.m_a[2][2] = T.m_right.m_a[2][2];
		T.m_b[0] = T.m_left.m_b[1];
		T.m_b[1] = T.m_left.m_b[2] + T.m_right.m_b[1];
		T.m_b[2] = T.m_right.m_b[2];
		return T;
	}
}

class A2Prim extends Production {
	A2Prim(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("A2Prim");
		T.m_a[0][0] = T.m_left.m_a[2][2] + T.m_right.m_a[1][1];
		T.m_a[1][0] = T.m_left.m_a[1][2];
		T.m_a[2][0] = T.m_right.m_a[2][1];
		T.m_a[0][1] = T.m_left.m_a[2][1];
		T.m_a[1][1] = T.m_left.m_a[1][1];
		T.m_a[2][1] = 0.0;
		T.m_a[0][2] = T.m_right.m_a[1][2];
		T.m_a[1][2] = 0.0;
		T.m_a[2][2] = T.m_right.m_a[2][2];
		T.m_b[0] = T.m_left.m_b[2] + T.m_right.m_b[1];
		T.m_b[1] = T.m_left.m_b[1];
		T.m_b[2] = T.m_right.m_b[2];
		return T;
	}
}

class E2 extends Production {
	E2(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("E2");
		T.m_b[0] /= T.m_a[0][0];
		T.m_a[0][2] /= T.m_a[0][0];
		T.m_a[0][1] /= T.m_a[0][0];
		T.m_a[0][0] /= T.m_a[0][0];

		T.m_b[1] -= T.m_b[0] * T.m_a[1][0];
		T.m_a[1][2] -= T.m_a[0][2] * T.m_a[1][0];
		T.m_a[1][1] -= T.m_a[0][1] * T.m_a[1][0];
		T.m_a[1][0] -= T.m_a[0][0] * T.m_a[1][0];

		T.m_b[2] -= T.m_b[0] * T.m_a[2][0];
		T.m_a[2][2] -= T.m_a[0][2] * T.m_a[2][0];
		T.m_a[2][1] -= T.m_a[0][1] * T.m_a[2][0];
		T.m_a[2][0] -= T.m_a[0][0] * T.m_a[2][0];

		return T;
	}
}

class ERoot extends Production {
	ERoot(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("ERoot");
		/*
		 * T.m_b[0]/=T.m_a[0][0]; T.m_a[0][2]/=T.m_a[0][0];
		 * T.m_a[0][1]/=T.m_a[0][0]; T.m_a[0][0]/=T.m_a[0][0];
		 * 
		 * T.m_b[1]-=T.m_b[0]*T.m_a[1][0]; T.m_a[1][2]-=T.m_a[0][2]*T.m_a[1][0];
		 * T.m_a[1][1]-=T.m_a[0][1]*T.m_a[1][0];
		 * T.m_a[1][0]-=T.m_a[0][0]*T.m_a[1][0];
		 * 
		 * T.m_b[2]-=T.m_b[0]*T.m_a[2][0]; T.m_a[2][2]-=T.m_a[0][2]*T.m_a[2][0];
		 * T.m_a[2][1]-=T.m_a[0][1]*T.m_a[2][0];
		 * T.m_a[2][0]-=T.m_a[0][0]*T.m_a[2][0];
		 */
		// root starts here
		T.m_b[1] /= T.m_a[1][1];
		T.m_a[1][2] /= T.m_a[1][1];
		T.m_a[1][1] /= T.m_a[1][1];

		T.m_b[2] -= T.m_b[1] * T.m_a[2][1];
		T.m_a[2][2] -= T.m_a[1][2] * T.m_a[2][1];
		T.m_a[2][1] -= T.m_a[1][1] * T.m_a[2][1];

		T.m_b[2] /= T.m_a[2][2];
		T.m_a[2][2] /= T.m_a[2][2];

		// 3rd row ok^^

		T.m_b[1] -= T.m_b[2] * T.m_a[1][2];
		T.m_a[1][2] -= T.m_a[2][2] * T.m_a[1][2];

		T.m_b[1] /= T.m_a[1][1]; // prawdodobnie niekonieczne bo
									// m_a[1][1]powinno by=1
		T.m_a[1][1] /= T.m_a[1][1];

		// 2nd row ok ^^

		T.m_b[0] -= T.m_b[2] * T.m_a[0][2];
		T.m_a[0][2] -= T.m_a[2][2] * T.m_a[0][2];

		T.m_b[0] -= T.m_b[1] * T.m_a[0][1];
		T.m_a[0][1] -= T.m_a[1][1] * T.m_a[0][1];

		T.m_b[0] /= T.m_a[0][0];// prawdodobnie niekonieczne bo m_a[0][0]powinno
								// by=1
		T.m_a[0][0] /= T.m_a[0][0];

		return T;
	}
}

class BS extends Production {
	BS(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("BS");
		if ("leaf" == T.m_label)
			return T;

		T.m_left.m_a[1][0] = 0.0;
		T.m_left.m_a[1][1] = 1.0;
		T.m_left.m_a[1][2] = 0.0;
		T.m_left.m_b[1] = T.m_b[1];

		T.m_left.m_a[2][0] = 0.0;
		T.m_left.m_a[2][1] = 0.0;
		T.m_left.m_a[2][2] = 1.0;
		T.m_left.m_b[2] = T.m_b[0];

		T.m_right.m_a[1][0] = 0.0;
		T.m_right.m_a[1][1] = 1.0;
		T.m_right.m_a[1][2] = 0.0;
		T.m_right.m_b[1] = T.m_b[0];

		T.m_right.m_a[2][0] = 0.0;
		T.m_right.m_a[2][1] = 0.0;
		T.m_right.m_a[2][2] = 1.0;
		T.m_right.m_b[2] = T.m_b[2];

		// ^^podstawione do dzieci
		return T;
	}
}

class BSS extends Production {
	BSS(Vertex Vert, Counter Count) {
		super(Vert, Count);
	}

	Vertex apply(Vertex T) {
		// System.out.println("BSS");
		T.m_b[0] -= T.m_b[2] * T.m_a[0][2];
		T.m_a[0][2] -= T.m_a[2][2] * T.m_a[0][2];

		T.m_b[0] -= T.m_b[1] * T.m_a[0][1];
		T.m_a[0][1] -= T.m_a[1][1] * T.m_a[0][1];

		T.m_b[0] /= T.m_a[0][0];
		T.m_a[0][0] /= T.m_a[0][0];

		return T;
	}
}
